import pygame, random, thorpy
from pygame.math import Vector2 as V
import imgmanager, parameters

class DebrisGenerator(object):
    def __init__(self, samples, color=None, size0=None, max_i=300, copy=False,
                    imgmanagers=None):
        self.max_i = max_i
        self.color = color
        if copy:
            self.samples = samples
            self.imgmanagers = imgmanagers
        else:
            samples = [s.copy() for s in samples]
            self.samples = samples
            if isinstance(size0, tuple):
                self.size0 = {s:size0 for s in self.samples}
            elif size0 is None:
                self.size0 = {s:s.get_size() for s in self.samples}
            else:
                self.size0 = size0
            if color is not None:#for the moment colorkey is hardcoded to white, and color source to black
                color=(254,254,254) if color == (255,255,255) else color
                for s in self.samples:
                    thorpy.change_color_on_img_ip(  img=s,
                                                    color_source=(0,0,0),
                                                    color_target=color,
                                                    colorkey=(255,255,255))
            self.imgmanagers = {s:imgmanager.ImgManager(s) for s in self.samples}
        self.debris = []

    def generate(self, q, n, v_range, omega_range, angle_range):
        angle_range = (angle_range[0]*100, angle_range[1]*100)
        v_range = (v_range[0]*100, v_range[1]*100)
        for i in range(n):
            angle = random.randint(angle_range[0],angle_range[1])/100.
            velocity = random.randint(v_range[0],v_range[1])/100.
            omega = random.randint(omega_range[0],omega_range[1])
            v = V(0,velocity).rotate(angle)
##            print("     ==>", self.color, q)
            self.debris.append(Debris(V(q), v, omega, self))

    def kill_old_elements(self, domain):
        to_remove = []
        for d in self.debris:
            if d.i > self.max_i or not(d.rect.colliderect(domain)):
                to_remove.append(d)
##            parent.partial_blit(None, d.rect)
        for d in to_remove:
            self.debris.remove(d)

    def draw(self, surface):
        for d in self.debris:
            surface.blit(d.img, d.rect.topleft)

    def update_physics(self,dt):
        for d in self.debris:
            d.update_physics(dt)

    def add_to(self, vessel):
        vessel.fight.debris.append((self, vessel))

    def get_copy(self):
        gen = DebrisGenerator(samples=self.samples, imgmanagers=self.imgmanagers,
                                max_i=self.max_i, copy=True)
        gen.color = self.color
        return gen

class Debris(object):

    def __init__(self, q, v, omega, generator):
        self.q = q
        self.v = v #constant
        self.angle = random.randint(-360,359)
        self.omega = omega
        self.s = random.choice(generator.samples)
        self.generator = generator
        self.img = self.generator.imgmanagers[self.s].imgs[int(self.angle)]
        self.rect = self.img.get_rect()
        self.rect.center = self.q
        self.i = 0

    def update_physics(self,dt):
        self.angle = (self.angle+self.omega*dt)%360
        self.q += self.v*dt
        self.img = self.generator.imgmanagers[self.s].imgs[int(self.angle)]
        self.rect = self.img.get_rect()
        self.rect.center = self.q
        self.i += 1
