import pygame, random, thorpy
import debris, smoke, parameters

def get_debris_generator(color, size=None):
    samples = []
    if size is None: size = parameters.MAX_DEBRIS_SIZE
    for w in range(1,size):
        for h in range(1,size):
            s = pygame.Surface(((w+1)*1.5,)*2).convert()
            s.fill((255,255,255))
            r = pygame.Rect(0,0,w,h)
            r.center = s.get_rect().center
            pygame.draw.rect(s, color, r)
            samples.append(s)
    return debris.DebrisGenerator(samples, color)


def get_smokegen(n=15, color=(99,99,99), grow=0.4, i=2, prob=0.3, alpha0=100,
                    size0=None):
    smoke_image1 = thorpy.load_image("smoke.png", (255,255,255))
    return smoke.SmokeGenerator([smoke_image1],
                                 n=n,
                                 prob=prob,
                                 grow=grow,
                                 i=i,
                                 color=color,
                                 alpha0=alpha0,
                                 size0=size0)

def build_debris():
    return {  "small":get_debris_generator(parameters.C_SMALL, 3),
              "medium":get_debris_generator(parameters.C_MED),
              "big":get_debris_generator(parameters.C_BIG)}


def build_smokes(): #for asteroids
    return {  "small":get_smokegen(50, parameters.C_SMALL,
                                    size0=(2*parameters.R_SMALL,)*2),
              "medium":get_smokegen(50, parameters.C_MED,
                                     size0=(2*parameters.R_MED,)*2),
              "big":get_smokegen(50, parameters.C_BIG,
                                        size0=(2*parameters.R_BIG,)*2),
              #explosive
              "smalle":get_smokegen(100, parameters.C_SMALL, 0.1,
                                    size0=(2*parameters.R_SMALL,)*2),
              "mediume":get_smokegen(100, parameters.C_MED, 0.1,
                                     size0=(2*parameters.R_MED,)*2),
              "bige":get_smokegen(100, parameters.C_BIG, 0.1,
                                        size0=(2*parameters.R_BIG,)*2),
              #burning
              "smallb":get_smokegen(80, parameters.C_SMALL, 0.1,
                                    size0=(2*parameters.R_SMALL,)*2, alpha0=255),
              "mediumb":get_smokegen(80, parameters.C_MED, 0.1,
                                    size0=(2*parameters.R_MED,)*2, alpha0=255),
              "bigb":get_smokegen(80, parameters.C_MED, 0.1,
                                    size0=(2*parameters.R_MED,)*2, alpha0=255),
              #explosive-burning
              "smalleb":get_smokegen(120, parameters.C_SMALL, 0.2, alpha0=200,
                                    size0=(2*parameters.R_SMALL,)*2),
              "mediumeb":get_smokegen(120, parameters.C_MED, 0.2, alpha0=200,
                                     size0=(2*parameters.R_MED,)*2),
              "bigeb":get_smokegen(120, parameters.C_BIG, 0.2, alpha0=200,
                                        size0=(2*parameters.R_BIG,)*2),

              #Explosions
              "smallexplosion":get_smokegen(30,(0,0,0),0.5,i=4,prob=0.8,alpha0=255,
                                            size0=(4*parameters.R_SMALL,)*2),
              "mediumexplosion":get_smokegen(30,(0,0,0),0.5,prob=0.8,alpha0=255,
                                            size0=(4*parameters.R_MED,)*2),
              "bigexplosion":get_smokegen(30,(0,0,0),0.5,prob=0.8,alpha0=255,
                                            size0=(4*parameters.R_BIG,)*2)
              }