import thorpy, pygame
import predefined, parameters

debris = None

def load():
    global debris
    debris = predefined.build_debris()

class Objective(object):

    def __init__(self, body, bodyname, what, body2=None, bodyname2=None):
        self.body = body
        self.what = what
        self.body2 = body2
        self.bodyname = bodyname
        self.bodyname2 = bodyname2

    def completed(self, universe):
        if self.what == parameters.OBJ_DESTROY:
            return self.body.must_death
        if self.what == parameters.OBJ_SURVIVE:
            return not self.body.must_death
        if self.what == parameters.OBJ_DEORBIT:
            if self.body.must_death:
                return False
            if self.body2.must_death:
                return True
            return not universe.is_orbiting(self.body, self.body2)
        if self.what == parameters.OBJ_ORBIT:
            if self.body.must_death:
                return False
            if self.body2.must_death:
                return False
            return universe.is_orbiting(self.body, self.body2)
        raise Exception("Invalid objective completion")

    def get_text(self):
        if self.what == parameters.OBJ_DEORBIT:
            return " ".join(["Deorbit", self.bodyname, "from", self.bodyname2])
        elif self.what == parameters.OBJ_DESTROY:
            return " ".join(["Destroy", self.bodyname])
        elif self.what == parameters.OBJ_ORBIT:
            return " ".join(["Make", self.bodyname, "orbit around", self.bodyname2])
        elif self.what == parameters.OBJ_SURVIVE:
            return " ".join([self.bodyname, "must survive"])
        else:
            raise Exception("Invalid objective")

    def get_copy(self, u_original, u_copy):
        """Update the bodies so that they match new universe.
        The bodies matching is based on their positions in the bodies list.
        """
        body = None
        body2 = None
        for i,b in enumerate(u_original.bodies):
            if b is self.body:
                body = u_copy.bodies[i]
            elif b is self.body2:
                body2 = u_copy.bodies[i]
        obj = Objective(body, self.bodyname, self.what, body2, self.bodyname2)
        return obj

class AObjective(Objective):

    def __init__(self, body2, bodyname2):
        Objective.__init__(self, None, None, parameters.OBJ_ORBIT, body2, bodyname2)

    def completed(self, universe):
        for b in universe.bodies:
            if b.is_asteroid:
                if universe.is_orbiting(b, self.body2):
                    return True
        return False

    def get_text(self):
        return " ".join(["Make an asteroid orbit around", self.bodyname2])

    def get_copy(self, u_original, u_copy):
        """Update the bodies so that they match new universe.
        The bodies matching is based on their positions in the bodies list.
        """
        body2 = None
        for i,b in enumerate(u_original.bodies):
            if b is self.body2:
                body2 = u_copy.bodies[i]
        obj = AObjective(body2, self.bodyname2)
        return obj

class Scenario(object):
    current_id = 0

    def __init__(self, universe, objectives, name="Mission", text=""):
        self.universe = universe
        self.objectives = objectives
        params = {}
        params["current_y"] = parameters.GAME_RECT.h//2
        params["current_v"] = (3,0)
        params["godness"] = parameters.MAX_GODNESS
        self.parameters = params
        self.name = name
        self.text = text
        self.id = Scenario.current_id
        Scenario.current_id += 1

    def update_status(self, universe):
        return [o.completed(universe) for o in self.objectives]

    def get_e_objectives(self):
        elements = []
        for o in self.objectives:
            text = o.get_text()
            checker = thorpy.Checker.make(text)
            checker.set_font_size(parameters.FONT_SIZE_OBJECTIVES)
            checker._name_element.active = False
            checker.active = False
            stick(checker._name_element, checker.get_fus_rect(), "left", "right")
            checker._name_element.move((-3,0))
            elements.append(checker)
        if not elements:
            return None
        title = thorpy.make_text("Goals", 12, (0,0,255))
        line = thorpy.Line.make(150, "h")
        box = thorpy.Box.make([title,line]+elements)
        box.objs = elements
        return box

    def prepare_debris(self):
        for b in self.universe.bodies:
            if not b.debris:
                if b.color in debris:
                    b.debris = debris[color].get_copy()
                else:
                    b.debris = predefined.get_debris_generator(b.color)

    def load(self):
        self.prepare_debris()
        u = self.universe.get_copy()
        o = [obj.get_copy(self.universe, u) for obj in self.objectives]
        return u, o, self.parameters.copy()

    def get_copy(self):
        u, o , p = self.load()
        s = Scenario(u, o, self.name, self.text)
        s.parameters = p
        return s

##    def completed(self):
##        for o in self.objectives:

def stick(element, target, target_side, self_side, align=True):
    """Sides must be either 'top', 'bottom, 'left' or 'right'.
    This function moves self in order to make its <self_side> just next to
    target's <target_side>.

    Note that unless <align> = True, this does not move self along the
    orthogonal axis: e.g, stick_to(target_element, 'right', 'left') will
    move self such that self.left = target.right (using storers rects), but
    self.top might not be target.top. Then this is up to the user to move
    self on the vertical axis once self is sticked to target.

    <target> can either be an element, "screen" or a rect.
    """
    r = element.get_storer_rect()
    topleft = r.topleft
    size = r.size
    if target == "screen":
        W, H = functions.get_screen_size()
        t = Rect(0, 0, W, H)
    elif isinstance(target, pygame.Rect):
        t = target
    else:
        t = target.get_storer_rect()
    target_topleft = t.topleft
    target_size = t.size
    if target_side == "left":
        sx = topleft[0]
        tx = target_topleft[0]
        if self_side == "right":
            sx += size[0]
        element.move((tx - sx, 0))
    elif target_side == "right":
        sx = topleft[0]
        tx = target_topleft[0] + target_size[0]
        if self_side == "right":
            sx += size[0]
        element.move((tx - sx, 0))
    elif target_side == "left":
        sx = topleft[0]
        tx = target_topleft[0]
        if self_side == "right":
            sx += size[0]
        element.move((tx - sx, 0))
    elif target_side == "bottom":
        sy = topleft[1]
        ty = target_topleft[1] + target_size[1]
        if self_side == "bottom":
            sy += size[1]
        element.move((0, ty - sy))
    elif target_side == "top":
        sy = topleft[1]
        ty = target_topleft[1]
        if self_side == "bottom":
            sy += size[1]
        element.move((0, ty - sy))
    else:
        return
    if align:
        if target_side == "top" or target_side == "bottom":
            element.set_center((t.centerx, None))
        else:
            element.set_center((None, t.centery))
